/***************************************************************************
                          mapcat_main.cc  -  description
                             -------------------
    begin                : Fri Dec  6 15:19:06 PST 2002
    copyright            : (C) 2002 by Oleksiy Kohany
    email                : kohany@girinst.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mapcat.h"
#include "functions.h"

//*******************************************
void usage(char *exec)
{
    if(!exec)
    {
        return;
    }
    if(strrchr(exec, '/'))
    {
        exec = strrchr(exec, '/') + 1;
    }
    cerr << "NAME\n\t" << exec;
#ifdef VERSION

    cerr << " V" << VERSION;
#endif

    cerr << " - removes overlapings and joins close fragments on the map\n";
    cerr << "\nSYNOPSIS\n\t" << exec << " [in_map(-)][out_map][-nc][--help][--version]\n";
    cerr << "\t[-noself] [-noselfonly] [-bylength] [-minlen #]\n";
    cerr << "\nDESCRIPTION\n\tRemoves overlapings and joins fragments that logically\n";
    cerr << "\tcould be fragments of the same element on the map\n";
    cerr << "\nFLAGS\n";
    cerr << "\t-minlen #\n\t   Minimum allowed length of fragments reported in map.\n";
    cerr << "\n\t-minsim #\n\t   Minimum allowed similarity of fragments reported in map.\n";
    cerr << "\n\t-qut n|p\n\t   Query sequence is nucleotide/protein. Default is nucleotide.\n";
    cerr << "\n\t-dbt n|p\n\t   Database sequence is nucleotide/protein. Default is nucleotide.\n";
    cerr << "\nFLAGS\n";
    cerr << "\t-nc\n\t   do not combine.\n";
    cerr << "\t   The program will not append poly-A tail to ALU repeats.\n";
    cerr << "\n\t-noself\n\t   removes self-hits from the map before processing.\n";
    cerr << "\n\t-noselfonly\n\t   only removes self-hits from the map and exits.\n\n";
#ifdef PACKAGE_BUGREPORT

    cerr << "\nBUG REPORT\n\t" << PACKAGE_BUGREPORT << "\n";
#endif
#ifdef COPYRIGHT

    cerr << "\nCOPYRIGHT\n\t" << COPYRIGHT << "\n";
#endif
}

CMPVALS CMatch::CMPBY = SCORE;
//*******************************************
int main(int argc, char** argv)
{
    FILE *FIN, *FOUT;
    char** args = argv;
    CMatch::compareBy(TGTFROM);
    REPMAP rmap;
    bool doCombine = true;
    bool noSelf = false;
    bool noSelfOnly = false;
    bool isQUp = false;
    bool isDBp = false;
    int iMinLen = 21;
    float dMinSim = 0.2;
    ARGMAP amap;
    getflags(argv, amap, 5, "-noself", "-noselfonly", "-nc", "--help", "--version");
    getoptions(argv, amap, 4, "-qut", "-dbt", "-minlen", "-minsim");
    if(amap.find("-qut") != amap.end())
    {
        if(amap["-qut"] == "p" || amap["-qut"] == "P")
        {
            isQUp = true;
        }
    }
    if(amap.find("-dbt") != amap.end())
    {
        if(amap["-dbt"] == "p" || amap["-dbt"] == "P")
        {
            isDBp = true;
        }
    }
    if(amap.find("-minlen") != amap.end())
    {
        if(sscanf(amap["-minlen"].c_str(), "%d", &iMinLen) < 0 || iMinLen < 0)
        {
            fprintf(stderr, "Bad value of -minlen option.\n");
            exit(5);
        }
    }
    if(amap.find("-minsim") != amap.end())
    {
        if(sscanf(amap["-minsim"].c_str(), "%f", &dMinSim) < 0 || dMinSim < 0)
        {
            fprintf(stderr, "Bad value of -minsim option.\n");
            exit(6);
        }
    }
    if(amap.find("-nc") != amap.end())
    {
        doCombine = false;
    }
    if(amap.find("-noself") != amap.end())
    {
        noSelf = true;
    }
    if(amap.find("-noselfonly") != amap.end())
    {
        noSelfOnly = true;
    }
    if(amap.find("--help") != amap.end() || amap.find("--version") != amap.end())
    {
        usage(argv[0]);
        exit(0);
    }
    if((args = assigninput(FIN, argv)) == 0)
    {
        fprintf(stderr, "Failed to asign input.\n");
        usage(argv[0]);
        exit(10);
    }
    if(loadMap(FIN, rmap, (noSelf || noSelfOnly)) == 0)
    {
/*
       if(noSelf || noSelfOnly)
        {
            exit(0);
        }
        fprintf(stderr, "FIN = %p, stdin = %p\n", (void*)FIN, (void*)stdin);
        fprintf(stderr, "Failed to load map. Read %d records.\n", rmap.size());
        exit(20);
*/
    }

    if(FIN != stdin)
    {
        fclose(FIN);
    }
    if((args = assignoutput(FOUT, args)) == 0)
    {
        fprintf(stderr, "Failed to asign output.\n");
        usage(argv[0]);
        exit(30);
    }
    //=============================================
    int res = 0;
    if(!noSelfOnly)
    {
        res = catMapX(rmap, 0.999, doCombine, isQUp, isDBp, iMinLen, dMinSim);
    }
    printMap(FOUT, rmap);
    //=============================================
    return res;
}
