/***************************************************************************
                          mapcat.cc  -  description
                             -------------------
    begin                : Fri Dec  6 15:19:06 PST 2002
    copyright            : (C) 2002 by Oleksiy Kohany
    email                : kohany@girinst.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mapcat.h"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "constants.h"
#include "functions.h"
#include <map>
#include <list>
#include <string>
#include <algorithm>

//#define DEBUG

using namespace std;

//--------------------------------------
long CMatch::getCmpVal()
{
    switch (CMPBY)
    {
    case REPFROM:
        return getRepFrom();
    case REPTO:
        return getRepTo();
    case TGTFROM:
        return getTgtFrom();
    case TGTTO:
        return getTgtTo();
    case SIM:
        return long(100 * getMatchRate());
    case SCORE:
    default:
        return long(100 * m_dScore);
    }
}
//--------------------------------------
void CMatch::compareBy(CMPVALS V)
{
    CMPBY = V;
}
//--------------------------------------
bool CMatch::isDirect()
{
    //    return (strstr(m_sRepName, "@2") == 0);
    return m_bIsDirect;
}
//--------------------------------------
void CMatch::assign(const char *pRepName, const int iRepFrom, const int iRepTo,
                    const char *pTgtName, const int iTgtFrom, const int iTgtTo,
                    const bool isDirect, const double dScore, const double dMatchRate, const double dTranzRate)
{
    m_dScore = dScore;
    m_dMatchRate = dMatchRate;
    m_dTranzRate = dTranzRate;
    m_bIsDirect = isDirect;
    strncpy(m_sRepName, pRepName, MAXNAMESIZE);
    repSeg.assign(iRepFrom, iRepTo);
    strncpy(m_sTgtName, pTgtName, MAXNAMESIZE);
    tgtSeg.assign(iTgtFrom, iTgtTo);
}
//--------------------------------------
void CMatch::assign(CMatch another)
{
    m_dScore = another.getScore();
    m_dMatchRate = another.getMatchRate();
    m_dTranzRate = another.getTranzRate();
    m_bIsDirect = another.isDirect();
    strncpy(m_sRepName, another.getRepName(), MAXNAMESIZE);
    repSeg.assign(another.getRepFrom(), another.getRepTo());
    strncpy(m_sTgtName, another.getTgtName(), MAXNAMESIZE);
    tgtSeg.assign(another.getTgtFrom(), another.getTgtTo());
}

//--------------------------------------
const segment& CMatch::getRepSeg() const
{
    return repSeg;
}
//--------------------------------------
const segment& CMatch::getTgtSeg() const
{
    return tgtSeg;
}
//--------------------------------------
const char* CMatch::getRepName() const
{
    return m_sRepName;
}
//--------------------------------------
const char* CMatch::getTgtName() const
{
    return m_sTgtName;
}
//--------------------------------------
const int CMatch::getRepFrom() const
{
    return (int)repSeg.getFrom();
}
//--------------------------------------
const int CMatch::getTgtFrom() const
{
    return (int)tgtSeg.getFrom();
}
//--------------------------------------
const int CMatch::getRepTo() const
{
    return (int)repSeg.getTo();
}
//--------------------------------------
const int CMatch::getTgtTo() const
{
    return (int)tgtSeg.getTo();
}
//--------------------------------------
const double CMatch::getScore() const
{
    return m_dScore;
}
//--------------------------------------
const double CMatch::getMatchRate() const
{
    return m_dMatchRate;
}
//--------------------------------------
const double CMatch::getTranzRate() const
{
    return m_dTranzRate;
}
//--------------------------------------
void CMatch::setRepFrom(int iRepFrom)
{
    repSeg.setFrom(iRepFrom);
}
//--------------------------------------
void CMatch::setRepTo(int iRepTo)
{
    repSeg.setTo(iRepTo);
}
//--------------------------------------
void CMatch::setTgtFrom(int iTgtFrom)
{
    tgtSeg.setFrom(iTgtFrom);
}
//--------------------------------------
void CMatch::setTgtTo(int iTgtTo)
{
    tgtSeg.setTo(iTgtTo);
}
//--------------------------------------
int CMatch::repLength()
{
    return (int)repSeg.getLength();
}
//--------------------------------------
int CMatch::tgtLength()
{
    return (int)tgtSeg.getLength();
}
//--------------------------------------
double CMatch::getMatchCount()
{
    return m_dMatchRate * tgtLength();
}
//--------------------------------------
bool CMatch::closeTo(CMatch &match, bool isQUp, bool isDBp, double dCloseRate, int iMaxHole)
{
    int iDBx = 1;
    int iQUx = 1;
    if(isQUp)
    {
        if(isDBp)
        {
            iDBx = 1;
            iQUx = 1;
        }
        else
        {
            iDBx = 1;
            iQUx = 3;
        }
    }
    else
    {
        if(isDBp)
        {
            iDBx = 3;
            iQUx = 1;
        }
        else
        {
            iDBx = 1;
            iQUx = 1;
        }
    }
    if(strcmp(getRepName(), match.getRepName()) || strcmp(getTgtName(), match.getTgtName()))
    {
        return false;
    }
    if(isDirect() ^ match.isDirect())
    {
        return false;
    }
    if(repSeg.contains(match.getRepSeg()) || repSeg.inside(match.getRepSeg()) ||
            tgtSeg.contains(match.getTgtSeg()) || tgtSeg.inside(match.getTgtSeg()))
    {
        return false;
    }
    long iR1F, iR1T, iT1F, iT1T;
    long iR2F, iR2T, iT2F, iT2T;
    (getTgtFrom() < match.getTgtFrom())
    ?
    (iR1F = iQUx*getRepFrom(), iR1T = iQUx*getRepTo(), iT1F = iDBx*getTgtFrom(), iT1T = iDBx*getTgtTo(),
     iR2F = iQUx*match.getRepFrom(), iR2T = iQUx*match.getRepTo(), iT2F = iDBx*match.getTgtFrom(), iT2T = iDBx*match.getTgtTo())
    :
    (iR1F = iQUx*match.getRepFrom(), iR1T = iQUx*match.getRepTo(), iT1F = iDBx*match.getTgtFrom(), iT1T = iDBx*match.getTgtTo(),
     iR2F = iQUx*getRepFrom(), iR2T = iQUx*getRepTo(), iT2F = iDBx*getTgtFrom(), iT2T = iDBx*getTgtTo());


    //Target and (or) query fragments overlap
    if((repSeg.overlaps(match.getRepSeg()) && tgtSeg.overlaps(match.getTgtSeg())) ||
            (repSeg.overlaps(match.getRepSeg()) ^ tgtSeg.overlaps(match.getTgtSeg())))
    {
        if(isDirect())
        {
            if(fabs(double(iR2T - iR1F) / (iT2T - iT1F) - 1) <= dCloseRate)
            {
                if(iT1T-iT2F == 0 || (iR1T-iR2F)/(iT1T-iT2F) > 0)
                {
                    if(max(iR2F - iR1T, iT2F - iT1T) <= iMaxHole)
                    {
                        return true;
                    }
                }
            }
#ifdef DEBUG
            else
            {
                cerr << "Not close enough: ("<<iR2T<<"-"<<iR1F<< ")/("<<iT2T<<"-"<<iT1F<<")-1="<<fabs(double(iR2T - iR1F) / (iT2T - iT1F) - 1) << " > " << dCloseRate << "\n";
            }
#endif

        }
        else
        {
            if(fabs(double(iR1T - iR2F) / (iT2T - iT1F) - 1) <= dCloseRate)
            {
                if(iT1T-iT2F == 0 || (iR2T-iR1F)/(iT1T-iT2F) > 0)
                {
                    if(max(iR1F - iR2T, iT2F - iT1T) <= iMaxHole)
                    {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    //Target or query fragments overlap
    if(repSeg.overlaps(match.getRepSeg()) ^ tgtSeg.overlaps(match.getTgtSeg()))
    {
        if(isDirect())
        {
            if(fabs(double(iR2T - iR1F) / (iT2T - iT1F) - 1) <= dCloseRate)
            {
                if(max(iR2F - iR1T, iT2F - iT1T) <= iMaxHole)
                {
                    return true;
                }
            }
        }
        else
        {
            if(fabs(double(iR1T - iR2F) / (iT2T - iT1F) - 1) <= dCloseRate)
            {
                if(max(iR1F - iR2T, iT2F - iT1T) <= iMaxHole)
                {
                    return true;
                }
            }
        }
        return false;
    }

    //Target and query fragments don't overlap
    if(!(repSeg.overlaps(match.getRepSeg()) || tgtSeg.overlaps(match.getTgtSeg())))
    {
        if(isDirect())
        {
            if(fabs(double(iR2T - iR1F) / (iT2T - iT1F) - 1) <= dCloseRate)
            {
                if(max(iR2F - iR1T, iT2F - iT1T) <= iMaxHole)
                {
                    return true;
                }
            }
        }
        else
        {
            if(fabs(double(iR1T - iR2F) / (iT2T - iT1F) - 1) <= dCloseRate)
            {
                if(max(iR1F - iR2T, iT2F - iT1T) <= iMaxHole)
                {
                    return true;
                }
            }
        }
        return false;
    }
    return false;
}
//--------------------------------------
bool CMatch::overlaps(CMatch &match, double dOvlpRate)
{
    if(strcmp(getTgtName(), match.getTgtName()))
    {
        return false;
    }
    int iOvlpLength = (tgtLength() + match.tgtLength()) -
                      (max(getTgtTo(), match.getTgtTo()) - min(getTgtFrom(), match.getTgtFrom()) + 1);
    if(iOvlpLength > 0)
    {
        if((double(iOvlpLength)) / min(tgtLength(), match.tgtLength()) >= dOvlpRate)
        {
            return true;
        }
    }
    return false;
}
//--------------------------------------
void CMatch::joinSimple(CMatch &match)
{
    int iOldRLen = repLength();
    int iOldTLen = tgtLength();
    if(!tgtSeg.contains(match.getTgtSeg()))
    {
        if(tgtSeg.inside(match.getTgtSeg()))
        {
            assign(match.getRepName(), match.getRepFrom(), match.getRepTo(),
                   match.getTgtName(), match.getTgtFrom(), match.getTgtTo(),
                   match.isDirect(), match.getScore(), match.getMatchRate(), match.getTranzRate());
        }
        else
        {
            tgtSeg.add(match.getTgtSeg());
            setRepFrom(min(getRepFrom(), match.getRepFrom()));
            setRepTo(getRepFrom() + getTgtTo() - getTgtFrom());
            m_dMatchRate = (getMatchRate() * iOldTLen + match.getMatchRate() * match.tgtLength()) / (max(tgtLength(), iOldTLen + match.tgtLength()));
            m_dTranzRate = (getTranzRate() * iOldTLen + match.getTranzRate() * match.tgtLength()) / (max(tgtLength(), iOldTLen + match.tgtLength()));
#ifdef DEBUG

            cerr << "NewScore = (" << getScore() << " + " << match.getScore() << ") * " << repLength() << " / (" << iOldRLen << " + " << match.repLength() << ")=";
#endif

            m_dScore = repLength() * (getScore() + match.getScore()) / (iOldRLen + match.repLength());
#ifdef DEBUG

            cerr << m_dScore<<"\n";
#endif

        }
    }
}
//--------------------------------------
void CMatch::join(CMatch &match)
{
    int iOldRLen = repLength();
    int iOldTLen = tgtLength();
    if(!tgtSeg.contains(match.getTgtSeg()))
    {
        if(tgtSeg.inside(match.getTgtSeg()))
        {
            assign(match.getRepName(), match.getRepFrom(), match.getRepTo(),
                   match.getTgtName(), match.getTgtFrom(), match.getTgtTo(),
                   match.isDirect(), match.getScore(), match.getMatchRate(), match.getTranzRate());
        }
        else
        {
            repSeg.add(match.getRepSeg());
            tgtSeg.add(match.getTgtSeg());
            m_dMatchRate = (getMatchRate() * iOldTLen + match.getMatchRate() * match.tgtLength()) / (max(tgtLength(), iOldTLen + match.tgtLength()));
            m_dTranzRate = (getTranzRate() * iOldTLen + match.getTranzRate() * match.tgtLength()) / (max(tgtLength(), iOldTLen + match.tgtLength()));
#ifdef DEBUG

            cerr << "NewScore = (" << getScore() << " + " << match.getScore() << ") * " << repLength() << " / (" << iOldRLen << " + " << match.repLength() << ")=";
#endif

            m_dScore = repLength() * (getScore() + match.getScore()) / (iOldRLen + match.repLength());
#ifdef DEBUG

            cerr << m_dScore<<"\n";
#endif

        }
    }
}
//--------------------------------------
void CMatch::extendBy(CMatch &match)
{
    tgtSeg.add(match.getTgtSeg());
}
//--------------------------------------
void CMatch::eraseBy(CMatch &match)
{
    double remainingPortion = 0;
    bool deleteFront = false;
    bool deleteRear = false;
    if(getTgtFrom() <= match.getTgtFrom() && getTgtTo() >= match.getTgtTo())
    {
    	if(match.getTgtFrom() - getTgtFrom() > getTgtTo() - match.getTgtTo()) {
    		deleteFront = true;
    	}
    	else {
    		deleteRear = true;
    	}
    }
    if(deleteFront || (getTgtFrom() <= match.getTgtFrom() &&
            getTgtTo() <= match.getTgtTo() &&
            getTgtTo() >= match.getTgtFrom()))
    {
        remainingPortion = ((double)tgtLength() - (1.0 + getTgtTo() - match.getTgtFrom())) / tgtLength();
        m_dScore *= remainingPortion;
        setTgtTo(match.getTgtFrom() - 1);
        if(isDirect())
        {
            setRepTo(round(getRepTo() -  (1.0 - remainingPortion) * repLength()));
        }
        else
        {
            setRepFrom(round(getRepFrom() + (1.0 - remainingPortion) * repLength()));
        }
        return;
    }
    if(deleteRear || (getTgtTo() >= match.getTgtTo() &&
            getTgtFrom() <= match.getTgtTo() &&
            getTgtFrom() >= match.getTgtFrom()))
    {
        remainingPortion = ((double)tgtLength() - (1.0 + match.getTgtTo() - getTgtFrom())) / tgtLength();
        m_dScore *= remainingPortion;
        setTgtFrom(match.getTgtTo() + 1);
        if(isDirect())
        {
            setRepFrom(round(getRepFrom() + (1.0 - remainingPortion) * repLength()));
        }
        else
        {
            setRepTo(round(getRepTo() - (1.0 - remainingPortion) * repLength()));
        }
        return;
    }
}
//--------------------------------------
void CMatch::print(char *pcLine)
{
    sprintf(pcLine, "%s %d %d %s %d %d %c %.4f %.4f %.2f",
            m_sTgtName, getTgtFrom(), getTgtTo(), m_sRepName, getRepFrom(), getRepTo(),
            m_bIsDirect ? 'd' : 'c', m_dScore, getMatchRate(), getTranzRate());
}
//--------------------------------------
void CMatch::print(FILE *FOUT)
{
    print_ft(FOUT, m_sTgtName, getTgtFrom(), getTgtTo(), m_sRepName, getRepFrom(), getRepTo(),
             m_dScore, getMatchRate(), getTranzRate(), m_bIsDirect);
}
//--------------------------------------
void CMatch::print(ostream* os)
{
    print_ft(*os, m_sTgtName, getTgtFrom(), getTgtTo(), m_sRepName, getRepFrom(), getRepTo(),
             m_dScore, getMatchRate(), getTranzRate(), m_bIsDirect);
}
//--------------------------------------
ostream& operator << (ostream& os, CMatch& mt)
{
    mt.print(&os);
    return os;
}
//--------------------------------------
bool CMatch::operator == (CMatch &match)
{
    return getCmpVal() == match.getCmpVal();
}
bool CMatch::operator > (CMatch &match)
{
    return getCmpVal() > match.getCmpVal();
}
bool CMatch::operator < (CMatch &match)
{
    return getCmpVal() < match.getCmpVal();
}
bool CMatch::operator >= (CMatch &match)
{
    return getCmpVal() >= match.getCmpVal();
}
bool CMatch::operator <= (CMatch &match)
{
    return getCmpVal() <= match.getCmpVal();
}

//*******************************************
void sortMap(REPMAP &rmap)
{
    REPMAP::iterator rmptr;
    for(rmptr = rmap.begin(); rmptr != rmap.end(); rmptr++)
    {
        (*rmptr).second.sort();
    }
}
//*******************************************
int loadMap(FILE *FIN, REPMAP &rmap, bool noSelf)
{
    char line[MAXLINELENGTH];
    CMatch match;
    rmap.clear();
    while(fgets(line, MAXLINELENGTH, FIN))
    {
        if(match.read(line) == 0)
        {
            if(noSelf && match.isSelfMatch())
            {
                continue;
            }
            rmap[match.getTgtName()].push_back(match);
        }
        else
        {
            cerr << "ERROR Loading:\n" << line << "\n";
        }
    }
    sortMap(rmap);
    return rmap.size();
}
//*******************************************
void printMap(FILE *FOUT, REPMAP &rmap)
{
    REPMAP::iterator rmptr;
    MATCHLIST::iterator lptr;
    sortMap(rmap);
    for(rmptr = rmap.begin(); rmptr != rmap.end(); rmptr++)
    {
        for(lptr = (*rmptr).second.begin(); lptr != (*rmptr).second.end(); lptr++)
        {
            (*lptr).print(FOUT);
        }
    }
}
//*******************************************
bool isAlu(const char *pcRepName)
{
    if(strncasecmp(pcRepName, "ALU", min(3, (int)strlen(pcRepName))) == 0)
    {
        return true;
    }
    return false;
}
//*******************************************
bool arrange(MATCHLIST &list, MATCHLIST::iterator &srcptr)
{
    CMatch tmpSave;
    MATCHLIST::iterator lptr1 = srcptr;
    MATCHLIST::iterator lptr2 = srcptr;
    bool bRearranged = false;
    int itr = 0;
    for(lptr1++; lptr1 != list.end() && *lptr1 < *lptr2; lptr1++)
    {
        tmpSave.assign(*lptr2);
        (*lptr2).assign(*lptr1);
        (*lptr1).assign(tmpSave);
        lptr2 = lptr1;
        bRearranged = true;
        if(++itr > 100000)
        {
            cerr << "Over 100000 iterations. Quitting\n";
            break;
        }
    }
    return bRearranged;
}
//*******************************************
int catMapX(REPMAP &rmap, double iOvlpRate, bool doCombine, bool isQUp, bool isDBp, int iMinLen, float dMinSim)
{
    REPMAP::iterator rmptr;
    MATCHLIST::iterator lptr1, lptr2;
    if(isDBp)
    {
        iMinLen /= 3;
    }
    if(isDBp)
    {
        dMinSim /= 3;
    }
    //====================================================================
    // First loop
    //====================================================================
    for(rmptr = rmap.begin(); rmptr != rmap.end(); rmptr++)
    {
        for(lptr1 = (*rmptr).second.begin(); lptr1 != (*rmptr).second.end(); lptr1++)
        {
            for(lptr2 = lptr1, lptr2++; lptr2 != (*rmptr).second.end();)
            {
#ifdef DEBUG
                //            fprintf(stderr, "Loop 1:\n");
                //            fprintf(stderr, "\n");
                //            (*lptr1).print(stderr);
                //            (*lptr2).print(stderr);
#endif
                //Appending poly-A tail, if any, to the end of ALU in direct orientation
                if(doCombine && isAlu((*lptr1).getRepName()) && (*lptr1).isDirect())
                {
                    if(abs((*lptr1).getTgtTo() - (*lptr2).getTgtFrom()) < 40)
                    {
                        if(strstr((*lptr2).getRepName(), "(A)n") != 0)
                        {
                            (*lptr1).extendBy((*lptr2));
                            lptr2 = (*rmptr).second.erase(lptr2);
                            continue;
                        }
                    }
                }
                //Appending poly-A tail, if any, to the end of ALU in complementary orientation
                if(doCombine && isAlu((*lptr2).getRepName()) && !(*lptr2).isDirect())
                {
                    if(abs((*lptr1).getTgtTo() - (*lptr2).getTgtFrom()) < 40)
                    {
                        if(strstr((*lptr1).getRepName(), "(A)n") != 0)
                        {
                            (*lptr2).extendBy((*lptr1));
                            arrange((*rmptr).second, lptr2);

                            lptr1 = (*rmptr).second.erase(lptr1);
                            if(lptr1 != (*rmptr).second.begin())
                            {
                                lptr1--;
                            }
                            lptr2 = lptr1;
                            lptr2++;
                            continue;
                        }
                    }
                }
                //Joining together 2 simple repeats, if close to each other
                if(strcmp((*lptr1).getRepName(), (*lptr2).getRepName()) == 0 && strstr((*lptr1).getRepName(), ")n") != 0 && *(strstr((*lptr1).getRepName(), ")n") + 2) == '\0')
                {
                    if((*lptr1).overlaps(*lptr2, 0) || (abs(max((*lptr1).getTgtFrom(), (*lptr2).getTgtFrom()) - min((*lptr1).getTgtTo(), (*lptr2).getTgtTo())) < 6))
                    {
                        (*lptr1).joinSimple(*lptr2);
                        lptr2 = (*rmptr).second.erase(lptr2);
                        continue;
                    }
                }
                //Joining 2 fragments of the same repeat, if close enough
                if((*lptr1).closeTo(*lptr2, isQUp, isDBp))
                {
#ifdef DEBUG
                    //               fprintf(stderr, "=>Close to\n");
                    //               (*lptr1).print(stderr);
#endif
                    (*lptr1).join((*lptr2));
                    lptr2 = (*rmptr).second.erase(lptr2);
                    continue;
                }
                if((*lptr1).getTgtSeg().dist((*lptr2).getTgtSeg()) > MAXHOLE)
                {
                    break;
                }
                lptr2++;
            }
        }
    }
    //====================================================================
    // Second loop
    //====================================================================
    for(rmptr = rmap.begin(); rmptr != rmap.end(); rmptr++)
    {
        for(lptr1 = (*rmptr).second.begin(); lptr1 != (*rmptr).second.end(); lptr1++)
        {
            //Removing overlaping of 2 fragments by partially erasing one of them
            for(lptr2 = lptr1, lptr2++; lptr2 != (*rmptr).second.end();)
            {
#ifdef DEBUG
                fprintf(stderr, "\n");
                fprintf(stderr, "Loop 2:\n");
                (*lptr1).print(stderr);
                (*lptr2).print(stderr);
                cerr << ((*lptr1).overlaps(*lptr2, 0) ? "overlap" : "don't overlap") << ", " << (*lptr1).getTgtFrom() << ", dist = " << (*lptr1).getTgtSeg().dist((*lptr2).getTgtSeg()) << "\n";
#endif
                //Removing overlaping of 2 fragments by erasing one of them
                if((*lptr1).overlaps(*lptr2, 0))
                {
                    if((*lptr1).overlaps(*lptr2, iOvlpRate))
                    {
#ifdef DEBUG
                        fprintf(stderr, "=>Overlaps Over %f\n", iOvlpRate);
#endif

                        if(((*lptr1).getScore() == (*lptr2).getScore() && (*lptr1).getMatchRate() <= (*lptr2).getMatchRate()) || (*lptr1).getScore() < (*lptr2).getScore())
                        {
                            if((*lptr1).tgtLength() <= (*lptr2).tgtLength())
                            {
                                lptr1 = (*rmptr).second.erase(lptr1);
                                lptr2 = lptr1;
                                lptr2++;
                                continue;
                            }
                            else
                            {
                                (*lptr1).eraseBy(*lptr2);
                                if(arrange((*rmptr).second, lptr1))
                                {
                                    lptr2 = lptr1;
                                    lptr2++;
                                    continue;
                                }
                            }
                        }
                        else
                        {
                            if((*lptr1).tgtLength() >= (*lptr2).tgtLength())
                            {
                                lptr2 = (*rmptr).second.erase(lptr2);
                                continue;
                            }
                            else
                            {
                                (*lptr2).eraseBy(*lptr1);
                                if(arrange((*rmptr).second, lptr2))
                                {
                                    continue;
                                }
                            }
                        }
                    }
                }
                else
                {
                    break;
                }
                lptr2++;
            }
        }
    }

    //====================================================================
    // Third loop
    //====================================================================
    bool lptr1inc = true;
    for(rmptr = rmap.begin(); rmptr != rmap.end(); rmptr++)
    {
        for(lptr1 = (*rmptr).second.begin(); lptr1 != (*rmptr).second.end();)
        {
            lptr1inc = true;
            //Removing overlaping of 2 fragments by partially erasing one of them
            for(lptr2 = lptr1, lptr2++; lptr2 != (*rmptr).second.end();)
            {
                if(lptr1 == lptr2)
                {
                    lptr2++;
                    continue;
                }
#ifdef DEBUG
                fprintf(stderr, "Loop 3:\n");
                fprintf(stderr, "\n");
                (*lptr1).print(stderr);
                (*lptr2).print(stderr);
                cerr << (*lptr1).getTgtFrom() << ", dist = " << (*lptr1).getTgtSeg().dist((*lptr2).getTgtSeg()) << "\n";
#endif

                if((*lptr1).overlaps(*lptr2, 0))
                {
#ifdef DEBUG
                    fprintf(stderr, "=>Overlaps\n");
#endif

                    if((*lptr1).getTgtSeg().contains((*lptr2).getTgtSeg()))
                    {
#ifdef DEBUG
                        (*lptr1).print(stderr);
#endif

                        lptr2 = (*rmptr).second.erase(lptr2);
                        continue;
                    }
                    if((*lptr2).getTgtSeg().contains((*lptr1).getTgtSeg()))
                    {
                        lptr1 = (*rmptr).second.erase(lptr1);
#ifdef DEBUG

                        (*lptr2).print(stderr);
#endif

                        lptr1inc = false;
                        break;
                    }
                    if((*lptr1).getScore() < (*lptr2).getScore() ||
                            ((*lptr1).getScore() == (*lptr2).getScore() && (*lptr1).tgtLength() >= (*lptr2).tgtLength()))
                    {
                        (*lptr1).eraseBy(*lptr2);
                        if(arrange((*rmptr).second, lptr1) )
                        {
                            lptr1inc = false;
                            break;
                        }
                    }
                    else
                    {
                        (*lptr2).eraseBy(*lptr1);
                        if(arrange((*rmptr).second, lptr2) )
                        {
                            continue;
                        }
                    }
                }
                else
                {
                    break;
                }
                lptr2++;
            }
            int iLT = iMinLen;
            // Removing short fragments, that are most likely to be garbage
            if (strstr ((*lptr1).getRepName (), ")n") != 0)
            {
                iLT = 10;
            }
            if ((*lptr1).tgtLength() * (*lptr1).getMatchRate() < iLT || (*lptr1).getMatchRate() < dMinSim)
            {
#ifdef DEBUG
                cerr << (*lptr1).tgtLength() << " * " << (*lptr1).getMatchRate() << " = " << (*lptr1).tgtLength() * (*lptr1).getMatchRate() << "\n";
                cerr << "Erasing as insignificant: " << (*lptr1) << "\n";
#endif

                lptr1 = (*rmptr).second.erase(lptr1);
                continue;
            }
            if(lptr1inc)
            {
                lptr1++;
            }
        }
    }
    return 0;
}
/** No descriptions */
int CMatch::read(char* line)
{
    long iRepFrom, iRepTo, iTgtFrom, iTgtTo;
    bool isDirect;
    double dMatchRate, dTranzRate;
    double dScore;
    char sRepName[MAXNAMESIZE], sTgtName[MAXNAMESIZE];
    if(read_ft(line, sTgtName, iTgtFrom, iTgtTo, sRepName, iRepFrom, iRepTo, dScore, dMatchRate, dTranzRate, isDirect) >= 9)
    {
        assign(sRepName, iRepFrom, iRepTo, sTgtName, iTgtFrom, iTgtTo, isDirect, dScore, dMatchRate, dTranzRate);
        return 0;
    }
    return 1;
}
/** No descriptions */
bool CMatch::isSelfMatch() const
{
    //  if(strcmp(m_sRepName, m_sTgtName)==0 && getTgtFrom()==getRepFrom() && getTgtTo()==getRepTo()&&getMatchRate()==1.0) {
    if(strcmp(m_sRepName, m_sTgtName)==0 && getTgtFrom()==getRepFrom() && getTgtTo()==getRepTo())
    {
        return 1;
    }
    return 0;
}
